#include "stdafx.h"
#include <stdarg.h>
#include <tchar.h>
#include "clf_timer.h"

//--------------------------------------------------------------------------------------
clf_timer* get_global_timer()
{
	// Using an accessor function gives control of the construction order
	static clf_timer timer;
	return &timer;
}

//--------------------------------------------------------------------------------------
clf_timer::clf_timer()
{
	m_bTimerStopped     = true;
#ifdef _WIN32
	m_bUsingQPF         = false;
	m_llQPFTicksPerSec  = 0;

	m_llStopTime        = 0;
	m_llLastElapsedTime = 0;
	m_llBaseTime        = 0;

	// Use QueryPerformanceFrequency() to get frequency of timer.  
	LARGE_INTEGER qwTicksPerSec;
	m_bUsingQPF = (bool) (QueryPerformanceFrequency( &qwTicksPerSec ) != 0);
	m_llQPFTicksPerSec = qwTicksPerSec.QuadPart;
#else
#endif	
}


//--------------------------------------------------------------------------------------
void clf_timer::reset()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return;

	// Get either the current time or the stop time
	LARGE_INTEGER qwTime;
	if( m_llStopTime != 0 )
		qwTime.QuadPart = m_llStopTime;
	else
		QueryPerformanceCounter( &qwTime );

	m_llBaseTime        = qwTime.QuadPart;
	m_llLastElapsedTime = qwTime.QuadPart;
	m_llStopTime        = 0;
#else
	gettimeofday(&last, 0);
#endif
	m_bTimerStopped     = FALSE;
}


//--------------------------------------------------------------------------------------
void clf_timer::start()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return;

	// Get the current time
	LARGE_INTEGER qwTime;
	QueryPerformanceCounter( &qwTime );

	if( m_bTimerStopped )
		m_llBaseTime += qwTime.QuadPart - m_llStopTime;
	m_llStopTime = 0;
	m_llLastElapsedTime = qwTime.QuadPart;
#else
	gettimeofday(&last, 0);
#endif
	m_bTimerStopped = FALSE;
}


//--------------------------------------------------------------------------------------
void clf_timer::stop()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return;

	if( !m_bTimerStopped )
	{
		// Get either the current time or the stop time
		LARGE_INTEGER qwTime;
		if( m_llStopTime != 0 )
			qwTime.QuadPart = m_llStopTime;
		else
			QueryPerformanceCounter( &qwTime );

		m_llStopTime = qwTime.QuadPart;
		m_llLastElapsedTime = qwTime.QuadPart;
		m_bTimerStopped = TRUE;
	}
#else
	m_bTimerStopped = TRUE;
#endif
}


//--------------------------------------------------------------------------------------
void clf_timer::advance()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return;

	m_llStopTime += m_llQPFTicksPerSec/10;
#endif
}


//--------------------------------------------------------------------------------------
double clf_timer::get_absolute_time()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return -1.0;

	// Get either the current time or the stop time
	LARGE_INTEGER qwTime;
	if( m_llStopTime != 0 )
		qwTime.QuadPart = m_llStopTime;
	else
		QueryPerformanceCounter( &qwTime );

	double fTime = qwTime.QuadPart / (double) m_llQPFTicksPerSec;

	return fTime;
#else
	return -1.0;
#endif
}


//--------------------------------------------------------------------------------------
double clf_timer::get_time()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return -1.0;

	// Get either the current time or the stop time
	LARGE_INTEGER qwTime;
	if( m_llStopTime != 0 )
		qwTime.QuadPart = m_llStopTime;
	else
		QueryPerformanceCounter( &qwTime );

	double fAppTime = (double) ( qwTime.QuadPart - m_llBaseTime ) / (double) m_llQPFTicksPerSec;

	return fAppTime;
#else
	struct timeval lcurrent;    // Current timer
	dFloat fSeconds, fFraction;  // Number of seconds & fraction (microseconds)

	// Get current time
	gettimeofday(&lcurrent, 0);

	// Subtract the last time from the current time. This is tricky because
	// we have seconds and microseconds. Both must be subtracted and added
	// together to get the final difference.
	sSeconds = (dFloat)(lcurrent.tv_sec - last.tv_sec);
	fFraction = (dFloat)(lcurrent.tv_usec - last.tv_usec);
	last = lcurrent;
	return fSeconds + fFraction;
#endif
}


//--------------------------------------------------------------------------------------
double clf_timer::get_elapsed_time()
{
#ifdef _WIN32
	if( !m_bUsingQPF )
		return -1.0;

	// Get either the current time or the stop time
	LARGE_INTEGER qwTime;
	if( m_llStopTime != 0 )
		qwTime.QuadPart = m_llStopTime;
	else
		QueryPerformanceCounter( &qwTime );

	double fElapsedTime = (double) ( qwTime.QuadPart - m_llLastElapsedTime ) / (double) m_llQPFTicksPerSec;
	m_llLastElapsedTime = qwTime.QuadPart;

	return fElapsedTime;
#else
	return get_time();
#endif
}


//--------------------------------------------------------------------------------------
bool clf_timer::is_stopped()
{
	return m_bTimerStopped;
}
